/*
                Sun Public License Notice

The contents of this file are subject to the Sun Public License
Version 1.0 (the "License"). You may not use this file except in
compliance with the License. A copy of the License is available at
http://www.sun.com/

The Original Code is NetBeans. The Initial Developer of the Original
Code is Sun Microsystems, Inc. Portions Copyright 1997-2004 Sun
Microsystems, Inc. All Rights Reserved.
*/
package org.netbeans.modules.snipe;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.AbstractAction;
import javax.swing.text.DefaultEditorKit;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;

import org.openide.actions.*;
import org.openide.awt.UndoRedo;
import org.openide.util.SharedClassObject;
import org.openide.util.Utilities;
import org.openide.windows.OutputWriter;
import org.openide.windows.IOProvider;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.TopComponent;

/** 
 * A component for displaying/editing snipes.
 *
 * @author  Rich Unger
 */
public class SnipeTopComponent extends CloneableTopComponent 
{
    protected SnipeEditor m_snipeEditor;

    public SnipeTopComponent() 
    {
        super();
        initComponents();
    }
    
    public SnipeTopComponent(SnipeDataObject dobj)
    {
        super();
        initComponents();
        initData(dobj);
    }

    protected void initData(SnipeDataObject dobj)
    { 
        m_snipeEditor.setSnipe(dobj);
        TopComponent.NodeName.connect(this, dobj.getNodeDelegate());
    } 
    
    public SnipeEditor getSnipeEditor()
    {
        return m_snipeEditor;
    }

    public Image getIcon()
    {
        return Utilities.loadImage ("org/netbeans/modules/snipe/resources/snipe16.gif");
    }
    
    public void open()
    {
        super.open();
        if(m_snipeEditor != null)
            m_snipeEditor.open();
    }
    
    // PERSISTENCE
    // If you wish to keep any state between IDE restarts, put it here:
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException 
    {
        super.readExternal (in);
        SnipeDataObject snipe = (SnipeDataObject)in.readObject();
        initComponents();
        initData(snipe);
    }

    public void writeExternal (ObjectOutput out) throws IOException 
    {
        super.writeExternal (out);
        out.writeObject (m_snipeEditor.getSnipe());
    }

    public int getPersistenceType() { 
        return PERSISTENCE_ONLY_OPENED;
    } 

    protected String preferredId() { 
        return "snipe" + Math.random();
    } 
    
    /** To enable cloning of the top component */
    protected CloneableTopComponent createClonedObject()
    {
        return new SnipeTopComponent(m_snipeEditor.getSnipe());
    }

    /**
     * Called when the last instance of a SnipeTopComponent for a particular
     * SnipeDataObject is about to be closed.  If the object is not saved, the user
     * is presented with a dialog.  Then the SnipeDataObject is closed and taken out of
     * memory.
     */
    protected boolean closeLast()
    {
        return m_snipeEditor.close();
    }

    protected void initComponents()
    { 
        // lay out the JPanel
        setLayout(new BorderLayout());
        m_snipeEditor = new SnipeEditor();
        add(m_snipeEditor.getPanel(), BorderLayout.CENTER);

        // hook into the netbeans Edit menu
        Action cutPerformer = new SnipeCutAction();
        Action copyPerformer = new SnipeCopyAction();
        Action pastePerformer = new SnipePasteAction();
        Action deletePerformer = new SnipeDeleteAction();

        ActionMap map = this.getActionMap();

        map.put(DefaultEditorKit.copyAction, copyPerformer);
        map.put(DefaultEditorKit.cutAction, cutPerformer);
        map.put(DefaultEditorKit.pasteAction, pastePerformer);
        map.put("delete", deletePerformer);
    } 

    public UndoRedo getUndoRedo() { 
        return new SnipeUndoRedo();
    } 

    private void log(String str) { 
        OutputWriter out = IOProvider.getDefault().getStdOut();
        out.println(str);
        out.flush();
    } 

    public class SnipeDeleteAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Delete the Snipe");
        } 
    } 
    
    public class SnipeCutAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Cut the Snipe");
        } 
    } 
    
    public class SnipeCopyAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Copy the Snipe");
        } 
    } 
    
    public class SnipePasteAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Paste the Snipe");
        } 
    } 

    public class SnipeUndoRedo extends UndoRedo.Manager { 
        public String getUndoPresentationName() { 
            return "snipe edit";
        } 

        public String getRedoPresentationName() { 
            return "snipe edit";
        } 

        public boolean canUndo() { 
            return true;
        } 

        public boolean canRedo() { 
            return true;
        } 

        public void undo() throws CannotUndoException { 
            log("Undo the Snipe");
        } 

        public void redo() throws CannotRedoException { 
            log("Redo the Snipe");
        } 
    } 
    
    private static final long serialVersionUID = -4406860224835839120L;
}
